@@ -8,7 +8,7 @@ Google Spreadsheets client library.
 
 """
 
-__version__ = '0.6.2'
+__version__ = '2.0.0'
 __author__ = 'Anton Burnashev'
 
 
@@ -18,9 +18,24 @@ except ImportError:
     from urllib.parse import urlencode
 
 
-from .client import Client, authorize
+from .client import Client
 from .models import Spreadsheet, Worksheet, Cell
 from .exceptions import (GSpreadException, AuthenticationError,
                          SpreadsheetNotFound, NoValidUrlKeyFound,
                          IncorrectCellLabel, WorksheetNotFound,
                          UpdateCellError, RequestError, CellNotFound)
+
+from . import v4
+
+
+def authorize(credentials, client_class=v4.Client):
+    """Login to Google API using OAuth2 credentials.
+    This is a shortcut function which
+    instantiates :class:`gspread.v4.client.Client`
+    and performs login right away.
+
+    :returns: :class:`gspread.v4.client.Client` instance.
+    """
+    client = client_class(auth=credentials)
+    client.login()
+    return client
