@@ -10,7 +10,6 @@ This module contains common spreadsheets' models
 
 from collections import defaultdict
 from itertools import chain
-from functools import wraps
 
 from xml.etree import ElementTree
 from xml.etree.ElementTree import Element, SubElement
@@ -18,13 +17,20 @@ from xml.etree.ElementTree import Element, SubElement
 from . import urlencode
 from .ns import _ns, _ns1, ATOM_NS, BATCH_NS, SPREADSHEET_NS
 from .urls import construct_url
-from .utils import finditem, numericise_all
-from .utils import rowcol_to_a1, a1_to_rowcol
 
-from .exceptions import (
-    IncorrectCellLabel, WorksheetNotFound, CellNotFound, ImportException
+from .utils import (
+    finditem,
+    numericise_all,
+    rowcol_to_a1,
+    a1_to_rowcol,
+    wid_to_gid,
+    cast_to_a1_notation
 )
 
+from .exceptions import WorksheetNotFound, CellNotFound
+
+from .base import BaseSpreadsheet, BaseCell
+
 try:
     unicode
 except NameError:
@@ -49,32 +55,7 @@ def _escape_attrib(text, encoding=None, replace=None):
 ElementTree._escape_attrib = _escape_attrib
 
 
-def cast_to_a1_notation(method):
-    """
-    Decorator function casts wrapped arguments to A1 notation
-    in range method calls.
-    """
-    @wraps(method)
-    def wrapper(self, *args, **kwargs):
-        try:
-            if len(args):
-                int(args[0])
-
-            # Convert to A1 notation
-            range_start = rowcol_to_a1(*args[:2])
-            range_end = rowcol_to_a1(*args[-2:])
-            range_name = ':'.join((range_start, range_end))
-
-            args = (range_name,) + args[4:]
-        except ValueError:
-            pass
-
-        return method(self, *args, **kwargs)
-
-    return wrapper
-
-
-class Spreadsheet(object):
+class Spreadsheet(BaseSpreadsheet):
     """ A class for a spreadsheet object."""
 
     def __init__(self, client, feed_entry):
@@ -209,65 +190,6 @@ class Spreadsheet(object):
         except IndexError:
             return None
 
-    def share(self, value, perm_type, role, notify=True, email_message=None):
-        """Share the spreadsheet with other accounts.
-        :param value: user or group e-mail address, domain name
-                      or None for 'default' type.
-        :param perm_type: the account type.
-               Allowed values are: ``user``, ``group``, ``domain``,
-               ``anyone``.
-        :param role: the primary role for this user.
-               Allowed values are: ``owner``, ``writer``, ``reader``.
-        :param notify: Whether to send an email to the target user/domain.
-        :param email_message: The email to be sent if notify=True
-
-        Example::
-
-            # Give Otto a write permission on this spreadsheet
-            sh.share('otto@example.com', perm_type='user', role='writer')
-
-            # Transfer ownership to Otto
-            sh.share('otto@example.com', perm_type='user', role='owner')
-
-        """
-        self.client.insert_permission(
-            self.id,
-            value=value,
-            perm_type=perm_type,
-            role=role,
-            notify=notify,
-            email_message=email_message
-        )
-
-    def list_permissions(self):
-        """Lists the spreadsheet's permissions.
-        """
-        return self.client.list_permissions(self.id)
-
-    def remove_permissions(self, value, role='any'):
-        """
-        Example::
-
-            # Remove Otto's write permission for this spreadsheet
-            sh.remove_permissions('otto@example.com', role='writer')
-
-            # Remove all Otto's permissions for this spreadsheet
-            sh.remove_permissions('otto@example.com')
-        """
-        permission_list = self.client.list_permissions(self.id)
-
-        key = 'emailAddress' if '@' in value else 'domain'
-
-        filtered_id_list = [
-            p['id'] for p in permission_list
-            if p[key] == value and (p['role'] == role or role == 'any')
-        ]
-
-        for permission_id in filtered_id_list:
-            self.client.remove_permission(self.id, permission_id)
-
-        return filtered_id_list
-
 
 class Worksheet(object):
 
@@ -279,6 +201,7 @@ class Worksheet(object):
         self._id = element.find(_ns('id')).text.split('/')[-1]
         self._title = element.find(_ns('title')).text
         self._element = element
+
         try:
             self.version = self._get_link(
                 'edit', element).get('href').split('/')[-1]
@@ -301,6 +224,11 @@ class Worksheet(object):
         """Title of a worksheet."""
         return self._title
 
+    @property
+    def gid(self):
+        """Gid of a worksheet."""
+        return wid_to_gid(self._id)
+
     @property
     def row_count(self):
         """Number of rows"""
@@ -369,7 +297,7 @@ class Worksheet(object):
 
         Example:
 
-        >>> wks.acell('A1') # this could be 'a1' as well
+        >>> worksheet.acell('A1')
         <Cell R1C1 "I'm cell A1">
 
         """
@@ -384,7 +312,7 @@ class Worksheet(object):
 
         Example:
 
-        >>> wks.cell(1, 1)
+        >>> worksheet.cell(1, 1)
         <Cell R1C1 "I'm cell A1">
 
         """
@@ -406,6 +334,16 @@ class Worksheet(object):
         :param last_row: Integer row number
         :param last_col: Integer row number
 
+        Example::
+
+            >>> # Using A1 notation
+            >>> worksheet.range('A1:B7')
+            [<Cell R1C1 "42">, ...]
+
+            >>> # Same with numeric boundaries
+            >>> worksheet.range(1, 1, 7, 2)
+            [<Cell R1C1 "42">, ...]
+
         """
         feed = self.client.get_cells_feed(
             self,
@@ -494,8 +432,7 @@ class Worksheet(object):
 
         Example:
 
-        >>> wks.update_acell('A1', '42') # this could be 'a1' as well
-        <Cell R1C1 "I'm cell A1">
+            worksheet.update_acell('A1', '42') # this could be 'a1' as well
 
         """
         return self.update_cell(*(a1_to_rowcol(label)), val=val)
@@ -507,6 +444,10 @@ class Worksheet(object):
         :param col: Column number.
         :param val: New value.
 
+        Example::
+
+            worksheet.update_cell(1, 1, '42')
+
         """
         feed = self.client.get_cells_cell_id_feed(self,
                                                   self._cell_addr(row, col))
@@ -550,6 +491,17 @@ class Worksheet(object):
 
         :param cell_list: List of a :class:`Cell` objects to update.
 
+        Example::
+
+            # Select a range
+            cell_list = worksheet.range('A1:C7')
+
+            for cell in cell_list:
+                cell.value = 'O_o'
+
+            # Update in batch
+            worksheet.update_cells(cell_list)
+
         """
         feed = self._create_update_feed(cell_list)
         self.client.post_cells(self, ElementTree.tostring(feed))
@@ -747,7 +699,7 @@ class Worksheet(object):
         self.update_cells(cells)
 
 
-class Cell(object):
+class Cell(BaseCell):
     """An instance of this class represents a single cell
     in a :class:`worksheet <Worksheet>`.
 
@@ -758,25 +710,12 @@ class Cell(object):
         cell_elem = element.find(_ns1('cell'))
         self._row = int(cell_elem.get('row'))
         self._col = int(cell_elem.get('col'))
+
+        #: Raw value of the cell (e.g. formula)
         self.input_value = cell_elem.get('inputValue')
+
         numeric_value = cell_elem.get('numericValue')
         self.numeric_value = float(numeric_value) if numeric_value else None
 
         #: Value of the cell.
         self.value = cell_elem.text or ''
-
-    @property
-    def row(self):
-        """Row number of the cell."""
-        return self._row
-
-    @property
-    def col(self):
-        """Column number of the cell."""
-        return self._col
-
-    def __repr__(self):
-        return '<%s R%sC%s %s>' % (self.__class__.__name__,
-                                   self.row,
-                                   self.col,
-                                   repr(self.value))
